from pklib.DataEncoder import *
from pklib.DataScaling import *
from pklib.DataVisualization import *
from pklib.ExploreData import *
from pklib.FeatureEngineering import *
from pklib.InputOptimization import *
from pklib.MissingValues import *
from pklib.Outlier import *
from pklib.Preprocessor import *
from pklib.PrincipalComponentAnalysis import *




import numpy as np
import pandas as pd
from pandas_profiling import ProfileReport


class Preprocessor:

    def __init__(self):
        """Constructor"""

    def import_data(self, path, file, missing_values=None, data_types=None):

        # Concatenating Path and File
        data_file = path + file
        df = pd.read_csv(data_file, na_values=missing_values,
                         low_memory=False,
                         dtype=data_types)
        return df

    def get_unique_values(self, df, column_list):
        list_unique = []
        for col in column_list:
            list_unique.append([col, df[col].nunique()])

        df = pd.DataFrame(list_unique)
        df.columns = ['feature', 'unique_items']
        df = df.sort_values(by='unique_items', ascending=False)
        return df

    def remove_constant_columns(self, df):
        """Automatically remove constant columns from the data set"""
        for col in df.columns:
            if df[col].nunique() == 1:
                """Drop"""
                df = df.drop([col], axis=1)

        return df

    def set_column_type(self, df, column_list, category):
        """Set Columns Data Type"""
        for col in df.columns:
            if col in column_list:
                df[col]= df[col].astype(category)
        return df

    def convert_object_to_category_columns(self, df):
        """Automatically convert object data types into Categorical data types"""
        df.loc[:, df.dtypes == 'object'] = df.select_dtypes(['object']).apply(lambda x: x.astype('category'))
        return  df

    def generate_profile_report(self, df, report_name):
        # Method Using Profiling
        profile = ProfileReport(df, minimal=True,
                                plot={'histogram': {'bins': 8}},
                                title='Hospital Care Report',
                                html={'style': {'full_width': True}})
        profile.to_file(output_file=report_name)

    def get_numerical_cols(self, df):
        """Collect numerical columns from the given dataset"""
        ncols = df.iloc[:, (np.where((df.dtypes == np.int64) | (df.dtypes == np.float64)))[0]].columns
        return ncols

    def validate_data_type(self, df, column_list):
        """Automatically categorize the columns by it unique counts"""
        list_unique = []
        for col in column_list:
            data_type = 'object'
            if df[col].nunique() > 2:
                if df[col].dtype == 'int64':
                    data_type = 'numerical'
                else:
                    data_type = 'categorical'
            elif df[col].nunique() > 1:
                data_type = 'binary'
            else:
                data_type = 'constant'

            list_unique.append([data_type, col, df[col].nunique()])

        df = pd.DataFrame(list_unique)
        df.columns = ['data_type', 'feature', 'unique_items']
        df = df.sort_values(by='unique_items', ascending=False)
        return df

    def get_binary_columns(self, df):
        """Collect numerical columns from the given dataset"""
        list_binary_cols = []
        for col in df.columns:
            data_type = 'object'
            if df[col].nunique() == 2:
                data_type = 'binary'
                list_binary_cols.append(col)

        return list_binary_cols

    def get_categorical_columns(self, df):
        """Collect categorical column from the given dataset"""
        list_categorical_cols = []
        for col in df.columns:
            data_type = 'object'
            if df[col].nunique() > 2:
                if not df[col].dtype == 'int64':
                    data_type = 'categorical'
                    list_categorical_cols.append(col)

        return list_categorical_cols












from palettable.matplotlib import matplotlib

# Import Libraries
import pandas as pd
import numpy as np


import matplotlib as mtl
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import missingno as msno
import seaborn as sns

from sklearn import preprocessing
from sklearn.compose import ColumnTransformer
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split
from sklearn.pipeline import Pipeline
from sklearn.impute import SimpleImputer

from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import MinMaxScaler
from sklearn.preprocessing import RobustScaler


class MissingValues:
    """Missing Values Operations"""

    def __init__(self):
        """Constructor"""

    def generate_missing_value_plots(self, df, title, path):
        """Visualize the missing plots"""
        self.visualize_null_plot(df, title, path)
        self.visualize_missing_density(df, title, path)
        self.visualize_missing_bar(df, title, True, path)
        self.visualize_missing_bar(df, title, False, path)
        self.visualize_missing_heatmap(df, title, path)
        self.visualize_missing_dendrogram(df, title, path)

    def visualize_null_plot(self, df, title, path=None):
        """To Visualize the null values from DataFrame"""
        title = 'Pandas Missing Values ' + title
        df.isnull().sum().plot(kind='bar')
        plt.title(title, fontsize=20, color='#3dcd58')
        file_name = path + title + '.png'
        plt.savefig(file_name, bbox_inches='tight', dpi=100, title=plt.title)
        plt.close()

    def visualize_missing_density(self, df, title, path=None):
        """To visualize the missing value density of given dataframe"""
        title = 'Missing Data Density ' + title
        msno.matrix(df)
        plt.title(title, fontsize=20, color='#3dcd58')
        file_name = path + title + '.png'
        plt.savefig(file_name, bbox_inches='tight', dpi=100, title=plt.title)
        plt.close()

    def visualize_missing_bar(self, df, title, log, path=None):
        """Visualize the missing data in Bar Chart"""
        title = 'Missing Data Bar ' + title
        msno.bar(df, color="blue", log=log)
        plt.title(title, fontsize=20, color='#3dcd58')
        file_name = path + title + '.png'
        plt.savefig(file_name, bbox_inches='tight', dpi=100, title=plt.title)
        plt.close()

    def visualize_missing_heatmap(self, df, title, path=None):
        """Visualize the missing data in Heatmap"""
        title = 'Heat Map Missing Data Correlation ' + title
        msno.heatmap(df)
        plt.title(title, fontsize=20, color='#3dcd58')
        file_name = path + title + '.png'
        plt.savefig(file_name, bbox_inches='tight', dpi=100, title=plt.title)
        plt.close()

    def visualize_missing_dendrogram(self, df, title, path=None):
        """Visualize the missing data in Dendrogram"""
        title = 'Dendrogram Missing Data ' + title
        msno.dendrogram(df)
        plt.title(title, fontsize=20, color='#3dcd58')
        file_name = path+ title + '.png'
        plt.savefig(file_name, bbox_inches='tight', dpi=100, title=plt.title)
        plt.close()

    def get_null_counts(self, df):
        """Tabulate the Null Counts"""
        nulls = df.isnull().sum()
        total_rows = df.shape[0]
        null_list = []
        for series in nulls.iteritems():
            if series[1] > 0:
                type = df[series[0]].dtype
                percentage = round((series[1] / total_rows) * 100, 2)
                null_list.append([series[0], series[1], percentage, type])
        dfNull = pd.DataFrame(null_list)
        dfNull.columns = ['Column', 'Missing Values', 'Missing %', 'Type']
        dfNullSorted = dfNull.sort_values(by='Missing Values', ascending=False)
        return dfNullSorted

    def get_missing_columns_by_value(self, df, percent):
        """Getting the List of columns values equal or more than given percent"""
        dfNull = self.get_null_counts(df)
        high_missing = dfNull[dfNull['Missing %'] <= percent].index
        dfNull.drop(high_missing, inplace=True)
        column_list = dfNull[dfNull.columns[0]].values
        return dfNull, column_list

    def remove_columns(self, df, columns_list):
        """Remove Columns from DataFrame"""
        df = df.drop(columns_list, axis=1)
        return df

    def impute_by_mean(self, df, column):
        """To Impute Missing Values by Mean"""
        imputer = SimpleImputer(missing_values=np.nan, strategy='mean')
        df[column] = imputer.fit_transform(df[[column]]).ravel()
        return df

    def impute_by_median(self, df, column):
        """To Impute Missing Values by Median"""
        imputer = SimpleImputer(missing_values=np.nan, strategy='median')
        df[column] = imputer.fit_transform(df[[column]]).ravel()
        return df

    def impute_by_most_frequent(self, df, column):
        """To Impute Missing Values by Median"""
        imputer = SimpleImputer(missing_values=np.nan, strategy='most_frequent')
        df[column] = imputer.fit_transform(df[[column]]).ravel()
        return df





def main():
    """Main Function"""
    print('-------------------------------------\n')










    # Outlier
    outlier_data = mv.find_outlier_zscore(df_missing_dropped, 3)
    print('\nOutlier Points:\n', outlier_data)
    print('-------------------------------------\n')





if __name__ == '__main__':
    main()



import category_encoders as ce
import numpy as np
import pandas as pd
from sklearn.preprocessing import LabelEncoder

from pklib.Preprocessor import Preprocessor


class DataEncoder:
    """Data Encoding Operations"""

    def __init__(self):
        """Constructor"""
        self.pp = Preprocessor()

    def do_target_encoding(self, df, columns, target, target_true_val):
        """Do Target Encoding for the given columns"""

        # Get Unique Values
        for col in columns:
            unique_vals = df[col].unique()

            for val in unique_vals:
                is_val = df[col] == val
                df_col = df[is_val]
                all_val_items = df_col.shape[0]

                is_true = df_col[target] == target_true_val
                is_true_df_col = df_col[is_true]
                true_val_items = is_true_df_col.shape[0]

                if all_val_items > 0:
                    mean = round(true_val_items / all_val_items, 5)
                else:
                    mean = 0

                df[col] = df[col].replace(val, mean)


            df[col] = pd.to_numeric(df[col])
        return df

    def do_label_target_encoding(self, df, columns, target):
        """Do Target Encoding for Given Columns - Here Using Label and Target Encoder used"""
        df_lable_encoding = self.do_label_encoding(df, columns)
        te = ce.TargetEncoder(cols=columns)
        data = te.fit_transform(df_lable_encoding, df_lable_encoding[target])
        return data

    def do_label_encoding(self, df, columns):
        """Do Label Encoding for given columns"""
        le = LabelEncoder()
        for col in columns:
            df[col] = le.fit_transform(df[col])

        df[col] = pd.to_numeric(df[col])
        return df

    def do_ordinal_encoding(self, df, column, order_list, start_no):
        """Do Ordinal Encoding By Given Order"""
        data = df.copy()
        for item in order_list:
            data[column] = data[column].replace(item, start_no)
            start_no = start_no + 1

        data[column] = pd.to_numeric(data[column])
        return data

    def do_onehot_encoding(self, df, columns):
        """Do OneHot Encoding for given columns"""
        onehotencoder = ce.OneHotEncoder(cols=columns)
        data = onehotencoder.fit_transform(df)
        feat_names = onehotencoder.get_feature_names()
        data.columns = feat_names
        return data

    def do_dummy_encoding(self, df, columns):
        """Do dummy encoding for given columns"""
        df_concate = df.copy()
        for col in columns:
            df_ohe = pd.get_dummies(df[col], prefix=col, drop_first=True)
            df_drop = df_concate.drop(columns=[col], axis=1)
            df_concate = pd.concat([df_drop, df_ohe], axis=1, sort=False)
        return df_concate



    def do_binary_encoding(self, df, column, true_value ):
        """Do Binary Encoding for given Column"""
        unique_vals = df[column].unique()
        for val in unique_vals:
            if val == true_value:
                df[column] = df[column].replace(val, 1)
            else:
                df[column] = df[column].replace(val, 0)

        #df[column] = pd.to_numeric(df[column])
        df[column] = df[column].astype(np.uint8)
        return df




import pandas as pd
import numpy as np

pd.set_option('mode.chained_assignment', None)


class InputOptimziation:
    '''Input Optimization Class'''

    def __init__(self):
        '''Constructor'''

    def load_files(self, csv_data):
        # 1. Load Diabetics Data
        missing_values = ['?', ' ']
        df_data = pd.read_csv(csv_data, na_values=missing_values, low_memory=False,
                              dtype={'payer_code': 'category'})

        # 2. Read ICD Data
        csv_icd = 'data/icd.csv'
        df_icd = pd.read_csv(csv_icd)

        return df_data, df_icd

    def make_revisit(self, df):
        """Add Revisit Count Column"""
        # df_revisit = df['patient_nbr'].value_counts()
        dfp = df.copy()
        # for index, val in df_revisit.iteritems():
        #     print(index, val)
        #     dfp = dfp.copy()
        #     patient_nbr = index
        #     filter_visit = dfp['patient_nbr'] == patient_nbr
        #     dfp['patient_nbr'][filter_visit] = val

        # self.data['RevisitCount'] = self.data.patient_nbr.groupby(self.data.patient_nbr).transform('count')
        dfp['RevisitCount'] = dfp.patient_nbr.groupby(df['patient_nbr']).transform('count')

        return dfp

    def drop_revisit(self, df):
        """Dropping revist more than 2"""
        df_revisit = df['patient_nbr'].value_counts()
        df_visit = df_revisit[df_revisit > 2]
        rows = df_visit.shape[0]
        filter_not_readmitted = df['readmitted'] != 'NO'

        dfp = df.copy()
        for num in range(0, rows):
            dfp = dfp.copy()
            patient_nbr = df_revisit[df_revisit > 2].index[num]
            filter_visit = dfp['patient_nbr'] == patient_nbr
            filter_not_readmitted = dfp['readmitted'] != 'NO'
            df_filter_visit = dfp[filter_visit & filter_not_readmitted][1:]
            dfp = dfp.drop(df_filter_visit.index)
            print(num, '/', rows)

        return dfp

    def fill_nan(self, data, column, value):
        """Fill Nan with 0"""
        data = data[column].fillna(value, inplace=True)
        return data

    def filter_rows(self, data, feature):
        """To filter numerical rows"""

        # 1. Filter Non-Numerical Rows
        filter_numeric = pd.to_numeric(data[feature], errors='coerce').notnull()
        df_num = data[filter_numeric]
        df_num[feature] = df_num[feature].astype(float)

        # 2. Filter Numerical Rows
        filter_non_numeric = pd.to_numeric(data[feature], errors='coerce').isnull()
        df_non_num = data[filter_non_numeric]

        return df_num, df_non_num

    def map_icd(self, icd, data, feature):
        """To map numerical rows"""

        # 1. Map Non-Numerical Rows
        df_num, df_non_num = self.filter_rows(data, feature)
        df_non_num[feature] = 'External Injury'

        frames = [df_num, df_non_num]
        data = pd.concat(frames)

        # 2. Map Numerical Rows
        for index, row in icd.iterrows():
            data_process = data.copy()
            # Read ICD inputs
            disease = row['disease']
            low = row['start']
            high = row['end']

            additional = str(row['additional']).split(',')
            additional = [float(i) for i in additional]

            # Filter Numerical
            df_num, df_non_num = self.filter_rows(data_process, feature)
            filter_low = df_num[feature] >= low
            filter_high = df_num[feature] <= high
            filter_additional = df_num[feature].isin(additional)
            df_num[feature][(filter_low & filter_high) | filter_additional] = disease

            frames = [df_num, df_non_num]
            data = pd.concat(frames)

        return data

    def remove_expired(self, data):
        # 7. Drop Expired Rows
        expired_list = [11, 19, 20, 21]
        filter_expired = ~data['discharge_disposition_id'].isin(expired_list)
        df_expired = data[filter_expired]

        return df_expired


def main():
    """Main Function"""

    # df_data, df_icd = load_files()
    # fill_nan(df_data)

    # df_data = map_icd(df_icd, df_data, 'diag_1')
    # df_data = map_icd(df_icd, df_data, 'diag_2')
    # df_data = map_icd(df_icd, df_data, 'diag_3')
    # df_data = make_revisit(df_data)
    # df_data = drop_revisit(df_data)
    # df_data = remove_expired(df_data)
    # df_data.to_csv('data/input_optimized.csv', index=False)
    # df_data.to_csv('data/train_input.csv', index=False)

    debug = 0


if __name__ == '__main__':
    main()



import pandas as pd
import matplotlib.pyplot as plt

from sklearn.decomposition import PCA


class PrincipalComponentAnalysis:
    """Principal Component Analysis """

    def __init__(self):
        """Constructor"""

    def execute(self, df, features, target, covariance_percentage, plot_path, graphs=False):
        """Execute PCA"""
        feat_range = len(features) - 1
        for num in range(2, feat_range):
            try:
                pca = PCA(n_components=num)
                col_range = num + 1
                col_names = ['component_' + str(i) for i in range(1, col_range)]
                principal_components = pca.fit_transform(features)
                df_principal = pd.DataFrame(data=principal_components, columns=col_names)
                df_final = pd.concat([df_principal, df[target]], axis=1)
                info_percent = round(pca.explained_variance_ratio_.sum() * 100, 2)
                print('Principal Components: ', num, 'information percentage: ', info_percent, '%')
                if info_percent > covariance_percentage:
                    if graphs:
                        self.generate_graph(df_final, target, col_names, plot_path)

                    return df_final

            except Exception as e:
                """Exception"""
                print(e)
                break

        return df_final

    def generate_graph(self, df, target, feature_list, path):
        """Visualize PCA"""
        targets = pd.DataFrame(df[target].value_counts())
        targets = targets.index.to_list()

        colors = ['b', 'g', 'r', 'c', 'm', 'y', 'k', 'w']
        for feature1 in feature_list:
            for feature2 in feature_list:
                if feature1 != feature2:
                    fig = plt.figure(figsize=(8, 8))
                    ax = fig.add_subplot(1, 1, 1)
                    ax.set_xlabel(feature1, fontsize=15)
                    ax.set_ylabel(feature2, fontsize=15)
                    ax.set_title('Components Covariance ', fontsize=20)
                    for target, color in zip(targets, colors):
                        indicesToKeep = df['readmitted'] == target
                        ax.scatter(df.loc[indicesToKeep, feature1]
                                   , df.loc[indicesToKeep, feature2]
                                   , c=color
                                   , s=50)
                    ax.legend(targets)
                    ax.grid()
                    title = 'PCA ' + feature1 + ' vs ' + feature2
                    file_name = path + title + '.png'
                    fig.savefig(file_name, bbox_inches='tight', dpi=100, title=plt.title)
                    plt.close(fig)



import pandas as pd
import numpy as np
import seaborn as sb
from ggplot import *
import matplotlib.pyplot as plt


class DataVisualization:
    """Visualize the Data"""

    def __init__(self):
        """Constructor"""


    def generate_count_plots(self, df, path):
        """Generate Count Plots"""
        plt.figure(figsize=(20, 10))
        for col in df.columns:
            title = col + ' Count Plot'
            title = title.replace('>', 'greater')
            title = title.replace('<', 'lesser')
            title = title.replace('/', '_')
            plot = qplot(x=col, data=df) + ggtitle(title)
            file_name = path + title + '.png'
            plot.save(file_name, dpi=300)
            plt.title(title, fontsize=14, color='#3dcd58')


    def generate_density_plots(self, df, color, path):
        """Generate Density Plots"""
        plt.figure(figsize=(20, 10))
        for col in df.columns:
            try:
                title = col + ' Density Plot'
                title = title.replace('>', 'greater')
                title = title.replace('<', 'lesser')
                title = title.replace('/', '_')

                plot = ggplot(df, aes(x=col, color=color)) + \
                       geom_density() + ggtitle(title)
                file_name = path + title + '.png'
                plot.save(file_name, dpi=300)
                plt.title(title, fontsize=14, color='#3dcd58')
            except Exception as e:
                print('Density Plot Error: ', col, ' ', e)

    def generate_violin_plots(self, df, target, color, path):
        """Generate Density Plots"""
        plt.figure(figsize=(20, 10))
        for col in df.columns:
            try:
                title = col + ' Violin Plot'
                title = title.replace('>', 'greater')
                title = title.replace('<', 'lesser')
                title = title.replace('/', '_')

                plot = ggplot(df, aes(x=col, y=target,
                                      color=color)) + \
                       geom_violin() + ggtitle(title)
                file_name = path + title + '.png'
                plot.save(file_name, dpi=300)
                plt.title(title, fontsize=14, color='#3dcd58')
            except Exception as e:
                print('Violin Plot Error: ', col, ' ', e)


    def generate_histograms(self, df, target, fill, path):
        """Generate Density Plots"""
        plt.figure(figsize=(20, 10))
        for col in df.columns:
            try:
                title = col + ' Histogram'
                title = title.replace('>', 'greater')
                title = title.replace('<', 'lesser')
                title = title.replace('/', '_')

                plot = ggplot(df, aes(x=col, y=target,
                                      fill=fill)) + \
                       geom_histogram() + ggtitle(title)
                file_name = path + title + '.png'
                plot.save(file_name, dpi=300)
                plt.title(title, fontsize=14, color='#3dcd58')
            except Exception as e:
                print('Histogram Error: ', col, ' ', e)

    def generate_jitter_plot(self, df, target,  path):
        """Generate Density Plots"""
        plt.figure(figsize=(20, 10))
        for feat1 in df.columns:
            for feat2 in df.columns:
                if feat1 != feat2:
                    try:
                        title = feat1 + '_' + feat2 + ' Jitter Plot'
                        title = title.replace('>', 'greater')
                        title = title.replace('<', 'lesser')
                        title = title.replace('/', '_')

                        plot = ggplot(df, aes(x=feat1, y=feat2, color=target)) + \
                               geom_jitter() + ggtitle(title)
                        file_name = path + title + '.png'
                        plot.save(file_name, dpi=300)
                        plt.title(title, fontsize=14, color='#3dcd58')
                    except Exception as e:
                        print('Jitter Plot Error: ', feat1, ' ', e)






import pandas as pd


class ExploreData:
    """Data Exploration"""

    def __init__(self, data, id_data):

        self.data = data
        self.id_data = id_data

        self.df = self.import_data()
        self.id_dfs = self.get_id_dfs()

    def import_data(self):
        """Import CSV Data"""
        try:
            df = pd.read_csv(self.data)
            return df
        except Exception as exp:
            err = self.errObj.handle_error(str(exp))
            print(str(err))

    def get_id_dfs(self):
        """Get list of id mappings in dataframe"""
        try:
            df = pd.read_csv(self.id_data, header=None)
            rows = df.to_numpy().tolist()

            ids = []
            id = []
            for row in rows:
                if not pd.isnull(row[0]):
                    id.append(row)
                else:
                    df = pd.DataFrame(id)
                    df.columns = df.iloc[0]
                    df = df[1:]
                    id = []
                    ids.append(df)

            df = pd.DataFrame(id)
            df.columns = df.iloc[0]
            df = df[1:]
            ids.append(df)
            return ids

        except Exception as exp:
            err = self.errObj.handle_error(str(exp))
            print(str(err))

    def get_mapped_ids(self, feature, list):
        """Get Ids of the list of Features"""
        ids = []
        for df in self.id_dfs:
            if df.columns[0] == feature:
                for item in list:
                    list_items = df[df[df.columns[1]] == item]
                    ids.append(int(list_items[feature].iloc[0]))
        return ids

    def drop_samples(self, df, feature, items_list):
        """Drop Samples"""
        drop_index_list = []
        for num in items_list:
            list_items = df[df[feature] == num].index.tolist()
            for item in list_items:
                if not item in drop_index_list:
                    drop_index_list.append(item)

        df.drop(drop_index_list, inplace=True)
        return df

    def calc_prevalence(self, y_actual):
        return round(sum(y_actual) / len(y_actual), 2)




import pandas as pd
import matplotlib.pyplot as plt


class FeatureEngineering:
    """Feature Engineering Operations"""

    def __init__(self):
        '''Constructor'''

    def has_null_values(self, df, column_list):
        status = True
        val = df[column_list].isnull().sum()
        sum = val.sum()
        if sum == 0:
            status = False

        return status

    def get_missing_value_columns(self, df):
        col_list = []
        val = df[df.columns].isnull().sum()
        for series in val.iteritems():
            if series[1] > 0:
                col_list.append(series[0])

        return col_list

    def fill_missing_value_with_unknown(self, df, column_list):
        missing_cols = self.get_missing_value_columns(df)
        for col in column_list:
            if col in missing_cols:
                df[col] = df[col].fillna('UNK')  # UNK - Unknown Value

    def get_items_count(self, df, feature, ascending):
        count_list = []
        val = df.groupby(feature).size()
        for series in val.iteritems():
            count_list.append([series[0], series[1]])

        df = pd.DataFrame(count_list)
        df.columns = ['item', 'count']
        df = df.sort_values(by='count', ascending=ascending)
        return df

    def get_top_items(self, df, feature, count, ascending):
        count_list = []
        val = df.groupby(feature).size()
        for series in val.iteritems():
            count_list.append([series[0], series[1]])

        df = pd.DataFrame(count_list)
        df.columns = ['item', 'count']
        df = df.sort_values(by='count', ascending=ascending)
        count_list = df['item'].values
        return count_list[:count]

    def add_categorical_column(self, df, column, bin_list, include_right):
        bin_values = pd.cut(x=df[column], bins=bin_list, right=include_right)
        bin_name = column + '_bin'
        df[bin_name] = bin_values
        return df

    def convert_to_binary_column(self, df, column):
        """Create new columns """
        binary_values = []
        for value in df[column]:
            if value == 0:
                binary_values.append(0)
            else:
                binary_values.append(1)

        df = df.drop(columns=[column], axis=1)
        df[column] = binary_values

        return df

    def get_unique_info(self, df, column):
        ndf = df.copy()
        unique_list = ndf[column].unique()
        column_values = ndf[column].to_list()
        total = df.shape[0]

        unique = []
        for item in unique_list:
            count = column_values.count(item)
            percent = round((count / total) * 100, 2)
            unique.append([item, count, percent])

        dfUnique = pd.DataFrame(unique)
        dfUnique.columns = ['item', 'Count', 'percent']

        return dfUnique

    def summerize_items_frequency(self, df, column_list, value_list):
        ndf = df.copy()
        total = df.shape[0]

        rows = []
        for column in column_list:
            column_values = ndf[column].to_list()

            status = True
            for val in value_list:
                if not val in column_values:
                    status = False

            if status:
                item_percent = []
                item_count = []
                for item in value_list:
                    count = column_values.count(item)
                    percent = round((count / total) * 100, 2)
                    item_percent.append(percent)
                    item_count.append(count)

                row = [column]
                for num in range(0, len(value_list)):
                    row.append(item_percent[num])

                for num in range(0, len(value_list)):
                    row.append(item_count[num])

                rows.append(row)

        df_summary = pd.DataFrame(rows)

        col_names = ['column']
        for val in value_list:
            percent = val + ' (%)'
            col_names.append(percent)

        for val in value_list:
            count = val + ' (Nos.)'
            col_names.append(count)

        df_summary.columns = col_names
        return df_summary

    def generate_frequency_distribution_graphs(self, df, target, path):
        """Visulize the frequency distribution charts"""
        for col in df.columns:
            title = 'Frequeny of ' + col + ' against ' + target
            pd.crosstab(df[col], df[target]).plot(kind='bar')
            plt.title(title)
            plt.xlabel(col)
            plt.ylabel(target)
            plt.title(title, fontsize=18, color='#626469')

            file_name = path + title + '.png'
            plt.savefig(file_name, bbox_inches='tight', dpi=100)
            plt.close()

            title = 'Frequeny of ' + target + ' against ' + col
            pd.crosstab(df[target], df[col]).plot(kind='bar')
            plt.title(title)
            plt.xlabel(col)
            plt.ylabel(target)
            plt.title(title, fontsize=18, color='#626469')

            file_name = path + title + '.png'
            plt.savefig(file_name, bbox_inches='tight', dpi=100)
            plt.close()


from collections import Counter

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sb
from pylab import rcParams
from sklearn.cluster import DBSCAN

rcParams['figure.figsize'] = 5, 4
sb.set_style('whitegrid')


class Outlier:
    """Outlier Object"""

    def __init__(self):
        """Constructor"""

    def get_outlier_by_zscore(self, df, columns, threshold):
        """Need to Update"""
        print("Dropping Outliers - ZScore")
        info = []
        df_out = df.copy()
        for col in columns:
            try:
                mean_1 = np.mean(df)
                std_1 = np.std(df)
                outliers = []
                for y in df:
                    z_score = (y - mean_1) / std_1
                    if np.abs(z_score) > threshold:
                        """df_out = df_out[df_out[column] < upper_bound]"""

            except:
                print(col)

        return outliers

    def drop_outliers_by_iqr(self, df, columns, drop_percent):
        """Drop Outlier Rows By IQR"""
        print('Processing outliers:')
        info = []
        df_out = df.copy()
        df_out_rows = pd.DataFrame()
        for column in columns:
            try:
                details = df_out[column].describe()
                count = details[0]
                mean = details[1]
                std = details[2]
                min = details[3]
                q1 = details[4]
                q2 = details[5]
                q3 = details[6]
                max = details[7]

                iqr = q3 - q1
                upper_bound = round(q3 + (1.5 * iqr), 5)
                lower_bound = round(q1 - (1.5 * iqr), 5)

                # Ourlier Rows
                df_upper_rows = df_out[df_out[column] > upper_bound]
                df_lower_rows = df_out[df_out[column] < lower_bound]

                total_rows = df_out.shape[0]

                df_drop_check = df_out.copy()
                df_drop_check = df_drop_check[df_drop_check[column] < upper_bound]
                df_drop_check = df_drop_check[df_drop_check[column] > lower_bound]

                rows_dropped = total_rows - df_drop_check.shape[0]
                outlier_percent = round((rows_dropped / total_rows) * 100, 3)

                if outlier_percent < drop_percent:
                    df_out = df_out[df_out[column] < upper_bound]
                    df_out = df_out[df_out[column] > lower_bound]
                    info.append(
                        [column, 'dropped', outlier_percent, rows_dropped, total_rows, lower_bound, upper_bound, iqr,
                         min, q1, q2, q3,
                         max, mean, std])

                    df_out_rows = pd.concat([df_upper_rows, df_lower_rows, df_out_rows])

                    debug = 0
                else:
                    info.append(
                        [column, 'not dropped', outlier_percent, rows_dropped, total_rows, lower_bound, upper_bound,
                         iqr, min, q1, q2, q3,
                         max, mean, std])


            except Exception as e:
                print(column, '\tError: ', e)

        print('Outlier Rows', df_out_rows.shape)
        df_out_rows.to_csv('data/outlier_rows.csv', index=False)

        df_drop_info = pd.DataFrame(info)
        df_drop_info.columns = ['Feature', 'Status', 'Percent', 'Dropped', 'Total Rows', 'Lower', 'Upper', 'IQR', 'Min',
                                'Q1',
                                'Q2', 'Q3', 'Max', 'Mean', 'Std']

        return df_out, df_drop_info

    def generate_outlier_graphs(self, df, columns, target, path):
        """"Generate Outlier Graphs for Visualization """
        for col in columns:
            try:
                fig = plt.figure(figsize=(20, 6))
                main_title = col + ' Outlier'
                fig.suptitle(main_title, fontsize=18, color='#304FFE')

                ax1 = fig.add_subplot(121)
                ax1.set_title('Univariate', fontdict={'fontsize': 15, 'fontweight': 'medium'})
                df_outlier = df.filter([col])
                df_outlier.boxplot(return_type='dict')

                ax2 = fig.add_subplot(122)
                ax2.set_title('Multivariate', fontdict={'fontsize': 15, 'fontweight': 'medium'})
                sb.boxplot(x=col, y=target, data=df, palette='hls')

                file_name = path + col + '_outlier.png'
                plt.savefig(file_name, bbox_inches='tight', dpi=100)
                plt.close()
            except:
                print(col)
                continue

    def detect_outliers_by_dbscan(self, df, columns, target, path, eps, min_samples):
        """Generate Outlier Graphs by DBSCAN"""
        print('Outliers DBSCAN')
        for col in columns:
            try:
                y = df[target].values.reshape(-1, 1)
                x = df[col].values.reshape(-1, 1)
                model = DBSCAN(eps=eps, min_samples=min_samples).fit(x)
                outliers_df = pd.DataFrame(x)
                print(Counter(model.labels_))
                print(outliers_df[model.labels_ == -1])

                fig = plt.figure()
                ax = fig.add_axes([.1, .1, .1, .1])
                colors = model.labels_
                ax.scatter(x[:, 2], x[:, 1], c=colors, s=120)
                ax.et_xlabel('Petal Length')
                plt.title('DBScan for Outlier Detection')

                file_name = path + col + '_DBSCAN.png'
                plt.savefig(file_name, bbox_inches='tight', dpi=100)
                plt.close()


            except:
                print(col)



import pandas as pd
from sklearn.preprocessing import StandardScaler, MinMaxScaler, RobustScaler


class DataScaling:
    """Scale Data Features"""

    def __init__(self):
        """Constructor"""

    def z_scale(self, df, columns):
        """Scale only the numerical columns using Standard Scaler"""
        df_z = df.copy()
        scaler = StandardScaler()
        for col in columns:
            scaled_col = scaler.fit_transform(df[col].values.reshape(-1, 1))
            df_z.drop(col, axis=1)
            df_z[col] = scaled_col
        return df_z

    def min_max_scale(self, df, columns):
        """Scale only the numerical columns using Min-Max Normalization"""
        df_m = df.copy()
        scaler = MinMaxScaler()
        for col in columns:
            scaled_col = scaler.fit_transform(df[col].values.reshape(-1, 1))
            df_m.drop(col, axis=1)
            df_m[col] = scaled_col
        return df_m

    def robust_scale(self, df, columns):
        """Scale only the numerical columns using Robust Normalization"""
        df_r = df.copy()
        scaler = RobustScaler()
        for col in columns:
            scaled_col = scaler.fit_transform(df[col].values.reshape(-1, 1))
            df_r.drop(col, axis=1)
            df_r[col] = scaled_col
        return df_r



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pklib",
    version="0.0.5",
    author="Parthiban Kannan",
    author_email="parthiban_kannan@outlook.com",
    description="Machine Learning Simplified Library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.

        'Programming Language :: Python :: 3.6',

    ],
    python_requires=">=3.6",
)
